/*
 * mm_radio_resource.h
 *
 * Copyright (c) 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Heechul Jeon <heechul.jeon@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __MM_RADIO_RESOURCE_H__
#define __MM_RADIO_RESOURCE_H__

#include <murphy/plugins/resource-native/libmurphy-resource/resource-api.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif

#define __MMRADIO_RESOURCE_WAIT_TIME	3

#define MMRADIO_GET_RESOURCE_LOCK(rm)	(&((mm_radio_resource_manager *)rm)->lock)
#define MMRADIO_RESOURCE_LOCK(rm)	(g_mutex_lock(MMRADIO_GET_RESOURCE_LOCK(rm)))
#define MMRADIO_RESOURCE_UNLOCK(rm)	(g_mutex_unlock(MMRADIO_GET_RESOURCE_LOCK(rm)))

#define MMRADIO_GET_RESOURCE_COND(rm)	(&((mm_radio_resource_manager *)rm)->cond)
#define MMRADIO_RESOURCE_WAIT(rm)	g_cond_wait(MMRADIO_GET_RESOURCE_COND(rm), MMRADIO_GET_RESOURCE_LOCK(rm)
#define MMRADIO_RESOURCE_WAIT_UNTIL(rm, end_time) \
	g_cond_wait_until(MMRADIO_GET_RESOURCE_COND(rm), MMRADIO_GET_RESOURCE_LOCK(rm), end_time)
#define MMRADIO_RESOURCE_SIGNAL(rm)	g_cond_signal(MMRADIO_GET_RESOURCE_COND(rm));


typedef enum {
	MM_RADIO_RESOURCE_TYPE_RADIO,
	MM_RADIO_RESOURCE_MAX
} mm_radio_resource_type_e;

typedef enum {
	MM_RADIO_RESOURCE_STATE_NONE,
	MM_RADIO_RESOURCE_STATE_INITIALIZED,
	MM_RADIO_RESOURCE_STATE_PREPARED,
	MM_RADIO_RESOURCE_STATE_ACQUIRED,
	MM_RADIO_RESOURCE_STATE_MAX,
} mm_radio_resource_state_e;

typedef struct {
	mrp_mainloop_t *mloop;
	mrp_res_context_t *context;
	mrp_res_resource_set_t *rset;
	mm_radio_resource_state_e state;
	GCond cond;
	GMutex lock;
	void *user_data;
	bool is_connected;
	bool by_rm_cb;
} mm_radio_resource_manager;

int mmradio_resource_manager_init(mm_radio_resource_manager *resource_manager, void *user_data);
int mmradio_resource_manager_deinit(mm_radio_resource_manager *resource_manager);
int mmradio_resource_manager_prepare(mm_radio_resource_manager *resource_manager, mm_radio_resource_type_e resource_type);
int mmradio_resource_manager_unprepare(mm_radio_resource_manager *resource_manager);
int mmradio_resource_manager_acquire(mm_radio_resource_manager *resource_manager);
int mmradio_resource_manager_release(mm_radio_resource_manager *resource_manager);
int mmradio_resource_manager_get_state(mm_radio_resource_manager *resource_manager, mm_radio_resource_state_e *state);
int mmradio_resource_wait_for_connection(mm_radio_resource_manager *resource_manager);

#ifdef __cplusplus
}
#endif

#endif /* __MM_RADIO_RESOURCE_H__ */
